@@ -7,28 +7,21 @@
 
 module Command.DropUnused where
 
-import Control.Monad.State (liftIO)
 import qualified Data.Map as M
-import System.Directory
-import Data.Maybe
 
+import Common.Annex
 import Command
-import Types
-import Messages
-import Locations
 import qualified Annex
 import qualified Command.Drop
-import qualified Command.Move
 import qualified Remote
 import qualified Git
 import Types.Key
-import Utility
 
 type UnusedMap = M.Map String Key
 
-command :: [Command]
-command = [repoCommand "dropunused" (paramRepeating paramNumber) seek
-	"drop unused file content"]
+def :: [Command]
+def = [dontCheck fromOpt $ command "dropunused" (paramRepeating paramNumber)
+	seek "drop unused file content"]
 
 seek :: [CommandSeek]
 seek = [withUnusedMaps]
@@ -41,8 +34,8 @@ withUnusedMaps params = do
 	unusedtmp <- readUnusedLog "tmp"
 	return $ map (start (unused, unusedbad, unusedtmp)) params
 
-start :: (UnusedMap, UnusedMap, UnusedMap) -> CommandStartString
-start (unused, unusedbad, unusedtmp) s = notBareRepo $ search
+start :: (UnusedMap, UnusedMap, UnusedMap) -> FilePath -> CommandStart
+start (unused, unusedbad, unusedtmp) s = search
 	[ (unused, perform)
 	, (unusedbad, performOther gitAnnexBadLocation)
 	, (unusedtmp, performOther gitAnnexTmpLocation)
@@ -62,24 +55,22 @@ perform key = maybe droplocal dropremote =<< Annex.getState Annex.fromremote
 		dropremote name = do
 			r <- Remote.byName name
 			showAction $ "from " ++ Remote.name r
-			next $ Command.Move.fromCleanup r True key
-		droplocal = Command.Drop.perform key (Just 0) -- force drop
+			ok <- Remote.removeKey r key
+			next $ Command.Drop.cleanupRemote key r ok
+		droplocal = Command.Drop.performLocal key (Just 0) -- force drop
 
-performOther :: (Git.Repo -> Key -> FilePath) -> Key -> CommandPerform
+performOther :: (Key -> Git.Repo -> FilePath) -> Key -> CommandPerform
 performOther filespec key = do
-	g <- Annex.gitRepo
-	let f = filespec g key
+	f <- fromRepo $ filespec key
 	liftIO $ whenM (doesFileExist f) $ removeFile f
 	next $ return True
 
 readUnusedLog :: FilePath -> Annex UnusedMap
 readUnusedLog prefix = do
-	g <- Annex.gitRepo
-	let f = gitAnnexUnusedLog prefix g
+	f <- fromRepo $ gitAnnexUnusedLog prefix
 	e <- liftIO $ doesFileExist f
 	if e
-		then return . M.fromList . map parse . lines
-			=<< liftIO (readFile f)
+		then M.fromList . map parse . lines <$> liftIO (readFile f)
 		else return M.empty
 	where
 		parse line = (head ws, fromJust $ readKey $ unwords $ tail ws)
