@@ -7,28 +7,21 @@
 
 module Command.SendKey where
 
-import Control.Monad.State (liftIO)
-import System.Exit
-
-import Locations
-import qualified Annex
+import Common.Annex
 import Command
-import Content
-import Utility
+import Annex.Content
 import Utility.RsyncFile
-import Messages
 
-command :: [Command]
-command = [repoCommand "sendkey" paramKey seek
+def :: [Command]
+def = [command "sendkey" paramKey seek
 	"runs rsync in server mode to send content"]
 
 seek :: [CommandSeek]
 seek = [withKeys start]
 
-start :: CommandStartKey
+start :: Key -> CommandStart
 start key = do
-	g <- Annex.gitRepo
-	let file = gitAnnexLocation g key
+	file <- fromRepo $ gitAnnexLocation key
 	whenM (inAnnex key) $
 		liftIO $ rsyncServerSend file -- does not return
 	warning "requested key is not present"
