@@ -7,31 +7,27 @@
 
 module Command.Unlock where
 
-import Control.Monad.State (liftIO)
-import System.Directory hiding (copyFile)
-
+import Common.Annex
 import Command
-import qualified Annex
-import Types
-import Messages
-import Locations
-import Content
+import Annex.Content
 import Utility.CopyFile
-import Utility
+import Utility.FileMode
 
-command :: [Command]
-command =
-	[ repoCommand "unlock" paramPath seek "unlock files for modification"
-	, repoCommand "edit" paramPath seek "same as unlock"
+def :: [Command]
+def =
+	[ c "unlock" "unlock files for modification"
+	, c "edit" "same as unlock"
 	]
+	where
+		c n = command n paramPaths seek
 
 seek :: [CommandSeek]
-seek = [withFilesInGit start]
+seek = [withFilesInGit $ whenAnnexed start]
 
 {- The unlock subcommand replaces the symlink with a copy of the file's
  - content. -}
-start :: CommandStartString
-start file = isAnnexed file $ \(key, _) -> do
+start :: FilePath -> (Key, Backend Annex) -> CommandStart
+start file (key, _) = do
 	showStart "unlock" file
 	next $ perform file key
 
@@ -41,12 +37,11 @@ perform dest key = do
 	
 	checkDiskSpace key
 
-	g <- Annex.gitRepo
-	let src = gitAnnexLocation g key
-	let tmpdest = gitAnnexTmpLocation g key
+	src <- fromRepo $ gitAnnexLocation key
+	tmpdest <- fromRepo $ gitAnnexTmpLocation key
 	liftIO $ createDirectoryIfMissing True (parentDir tmpdest)
 	showAction "copying"
-	ok <- liftIO $ copyFile src tmpdest
+	ok <- liftIO $ copyFileExternal src tmpdest
         if ok
                 then do
 			liftIO $ do
